
//! @file View.h
//! Some Comment
//! Some Comment
//! @author ME

#ifndef Vis_View
#define Vis_View

namespace Vis { class Matrix4x4; }
namespace Vis { class View; }
namespace Vis { class Viewport; }
namespace Vis { class ViewData; }
namespace Vis { class ViewEvent; }

#include <boost/signal.hpp>       // connections
#include <Visualization/Matrix.h> // member
#include <Visualization/Types.h>  // OriginID

//! View is the abstract interface that all views must adhere to. The
//! interface for this class should be kept lightweight so as not to penalize
//! children classes with pointless implementation or unusued memory.
//! This class plays the role of the "Component" in the decorator pattern.
//! This class also plays the role of the prototype in the prototype pattern
//! and has cloning functionality.
//! @ingroup framework
//! @see GoF-Decorator, GoF-Prototype
class Vis::View
{
public: // Members

    //! @name Constructors / Conversions / Destructors
    //@{
    
    //! Default constructor initializing members
    View(void);
    
    //! Virtual destructor to aid in polymorphism
    //! Any event locks performed by the view will be
    //! removed prior to deletion.
    virtual ~View(void);
    
    //@}

    //! @name Abstract Interface
    //@{
    
    //! Construct a clone of this object and return it.
    //! Clones will share their rendered data pointer, but
    //! not their children or other data.
    //! @return An auto_ptr to the newly allocated View. This object is
    //! owned by the caller, and will not be managed by the cloning class.
    virtual std::auto_ptr<View> clone(void) const;
    
    //! Access the bottom node of a view chain.  This is the node
    //! that provides the overall context for the view.
    //! @return A reference to the bottommost node of the current view.
    virtual const View& root(void) const;
    
    //! @return number of layers in the view
    virtual const int size(void) const;
    
    //! @return viewport for this view
    virtual const Viewport& viewport(void) const;
    
    //! Changes the position of the viewport
    //! @param x - desired x position (pixels) of lower left corner
    //! @param y - desired y position (pixels) of lower left corner
    //! @see Viewport::reposition()
    virtual void reposition(const int x, const int y);

    //! Changes the size of the viewport
    //! @param width - desired width of the viewport
    //! @param height - desired height of the viewport
    //! @see Viewport::resize()
    virtual void resize(const unsigned int width, const unsigned int height);
    
    //! Flip the view in the provided direction(s)
    //! @param x - if true, flip the view in the x direction
    //! @param y - if true, flip the view in the y direction
    virtual void flip(const bool x, const bool y);
    
    //! Rotate the view CCW by the given angle
    //! @param angle - angle in degrees to rotate the view
    virtual void rotate(const float angle);
    
    //! Scale the view by the given constant
    //! @param scale - constant by which the view should be multiplied
    virtual void zoom(const float scale);
    
    //! Shift the view in the x and y directions
    //! @param x - desired shift in the x direction (pixels)
    //! @param y - desired shift in the y direction (pixels)
    virtual void shift(const float x, const float y);
    
    //! Reset the view manipulation xform to the identity
    virtual void home(void);
    
    //@}
};

#endif // Vis_View
